@@ -4,7 +4,7 @@ Constants and global program state.
 ##############################################################################
 LICENSE = """\
 This file is part of pagekite.py.
-Copyright 2010-2016, the Beanstalks Project ehf. and Bjarni Runar Einarsson
+Copyright 2010-2017, the Beanstalks Project ehf. and Bjarni Runar Einarsson
 
 This program is free software: you can redistribute it and/or modify it under
 the terms of the  GNU  Affero General Public License as published by the Free
@@ -24,7 +24,7 @@ import random
 import time
 
 PROTOVER = '0.8'
-APPVER = '0.5.9.0'
+APPVER = '0.5.9.3'
 AUTHOR = 'Bjarni Runar Einarsson, http://bre.klaki.net/'
 WWWHOME = 'https://pagekite.net/'
 LICENSE_URL = 'http://www.gnu.org/licenses/agpl.html'
@@ -44,8 +44,8 @@ SERVICE_CERTS = ['b5p.us', 'frontends.b5p.us', 'pagekite.net', 'pagekite.me',
 
 # Places to search for the CA Certificate bundle
 OS_CA_CERTS = (
-  "/etc/ssl/certs/ca-certificates.crt",      # Debian/Ubuntu/Gentoo etc.
   "/etc/pki/tls/certs/ca-bundle.crt",        # Fedora/RHEL
+  "/etc/ssl/certs/ca-certificates.crt",      # Debian/Ubuntu/Gentoo etc.
   "/etc/ssl/ca-bundle.pem",                  # OpenSUSE
   "/etc/pki/tls/cacert.pem",                 # OpenELEC
   "/etc/ssl/cert.pem",                       # OpenBSD
@@ -75,7 +75,12 @@ LOOPBACK = {'FE': LOOPBACK_FE, 'BE': LOOPBACK_BE}
 # Re-evaluate our choice of frontends every 45-60 minutes.
 FE_PING_INTERVAL     = (45 * 60) + random.randint(0, 900)
 
-PING_INTERVAL        = 90
+# This is a global count of disconnect errors; we use this
+# to adjust the ping interval over time.
+DISCONNECT_COUNT = 0
+
+PING_INTERVAL_MIN    = 20
+PING_INTERVAL        = 116  # Not quite 2 minutes... :-)
 PING_INTERVAL_MOBILE = 1800
 PING_INTERVAL_MAX    = 1800
 PING_GRACE_DEFAULT   = 40
