@@ -4,6 +4,7 @@ The program manual!
 """
 import re
 import time
+import os
 
 from common import *
 from compat import ts_to_iso
@@ -196,6 +197,12 @@ MAN_OPT_BACKEND = ("""\
             the name of one of the 'built-in' providers, or a URL format
             string for ad-hoc updating.
 
+    --keepalive</b>=<a>N</a> __
+            Force traffic over idle tunnels every N seconds, to cope with
+            firewalls that kill idle TCP connections. Backend only: if set
+            to "auto" (the default), the interval will be adjusted
+            automatically in response to disconnects.
+
     --all           __Terminate early if any tunnels fail to register.
     --new           __Don't attempt to connect to any kites' old front-ends.
     --fingerpath</b>=<a>P</a>  __Path recipe for the httpfinger back-end proxy.
@@ -274,6 +281,7 @@ MAN_OPT_SYSTEM = ("""\
     --runas</b>=<a>U</a>:<a>G</a>    __Set UID:GID after opening our listening sockets.
     --pidfile</b>=<a>P</a>    __Write PID to the named file.
     --errorurl</b>=<a>U</a>   __URL to redirect to when back-ends are not found.
+    --errorurl</b>=<a>D:U</a> __Custom error URL for domain D.
 
     --selfsign __
             Configure the built-in HTTP daemon for HTTPS, first generating a
@@ -326,7 +334,7 @@ MAN_SECURITY = ("""\
     For more, please visit: <https://pagekite.net/support/security/>
 """)
 MAN_LICENSE = ("""\
-    Copyright 2010-2016, the Beanstalks Project ehf. and Bjarni R. Einarsson.
+    Copyright 2010-2017, the Beanstalks Project ehf. and Bjarni R. Einarsson.
 
     This program is free software: you can redistribute it and/or modify it
     under the terms of the GNU Affero General Public License as published by
@@ -456,12 +464,13 @@ def DOC():
 
 
 def MAN(pname=None):
+  lastchange = os.path.getmtime(sys.argv[0])
   man = ("""\
 .\\" This man page is autogenerated from the pagekite.py built-in manual.
 .TH PAGEKITE "1" "%s" "https://pagekite.net/" "Awesome Commands"
 .nh
 .ad l
-""") % ts_to_iso(time.time()).split('T')[0]
+""") % ts_to_iso(lastchange).split('T')[0]
   for h, section, text in MANUAL_TOC:
     man += ('.%s %s\n\n%s\n\n'
             ) % (h, h == 'SH' and section.upper() or section,
