@@ -5,7 +5,7 @@ incoming or outgoing network connections.
 ##############################################################################
 LICENSE = """\
 This file is part of pagekite.py.
-Copyright 2010-2016, the Beanstalks Project ehf. and Bjarni Runar Einarsson
+Copyright 2010-2017, the Beanstalks Project ehf. and Bjarni Runar Einarsson
 
 This program is free software: you can redistribute it and/or modify it under
 the terms of the  GNU  Affero General Public License as published by the Free
@@ -47,10 +47,11 @@ class Tunnel(ChunkParser):
   S_PROTOS = 3
   S_ADD_KITES = 4
   S_IS_MOBILE = 5
+  S_VERSION = 6
 
   def __init__(self, conns):
     ChunkParser.__init__(self, ui=conns.config.ui)
-    self.server_info = ['x.x.x.x:x', [], [], [], False, False]
+    self.server_info = ['x.x.x.x:x', [], [], [], False, False, None]
     self.Init(conns)
     # We want to be sure to read the entire chunk at once, including
     # headers to save cycles, so we double the size we're willing to
@@ -117,6 +118,7 @@ class Tunnel(ChunkParser):
           version = v
         if common.gYamon:
           common.gYamon.vadd('version-%s' % version, 1, wrap=10000000)
+        self.server_info[self.S_VERSION] = version
 
         for replace in conn.parser.Header(prefix+'-Replace'):
           if replace in self.conns.conns_by_id:
@@ -184,6 +186,8 @@ class Tunnel(ChunkParser):
       logi.append(('mobile', 'True'))
     if self.server_info[self.S_ADD_KITES]:
       logi.append(('add_kites', 'True'))
+    if self.server_info[self.S_VERSION]:
+      logi.append(('version', self.server_info[self.S_VERSION]))
 
     if bad:
       for backend in bad:
@@ -563,9 +567,29 @@ class Tunnel(ChunkParser):
     if chunk_headers:
       for ch in chunk_headers: sending.append('%s: %s\r\n' % ch)
     sending.append('\r\n')
-    sending.append(data)
 
-    return self.SendChunked(sending, zhistory=self.zhistory[sid])
+    # Small amounts of data we just send...
+    if len(data) <= 1024:
+      sending.append(data)
+      return self.SendChunked(sending, zhistory=self.zhistory[sid])
+
+    # Larger amounts we break into fragments to work around bugs in
+    # some of our small-buffered embedded clients. We aim for roughly
+    # one fragment per packet, assuming an MTU of 1500 bytes.
+    sending.append('')
+    frag_size = max(1024, 1400-len(''.join(sending)))
+    first = True
+    while data or first:
+      sending[-1] = data[:frag_size]
+      if not self.SendChunked(sending, zhistory=self.zhistory[sid]):
+        return False
+      data = data[frag_size:]
+      if first:
+        sending = ['SID: %s\r\n' % sid, '\r\n', '']
+        frag_size = max(1024, 1400-len(''.join(sending)))
+        first = False
+
+    return True
 
   def SendStreamEof(self, sid, write_eof=False, read_eof=False):
     return self.SendChunked('SID: %s\r\nEOF: 1%s%s\r\n\r\nBye!' % (sid,
@@ -686,6 +710,7 @@ class Tunnel(ChunkParser):
 
   # If a tunnel goes down, we just go down hard and kill all our connections.
   def ProcessEofRead(self):
+    common.DISCONNECT_COUNT += 1
     self.Die()
     return False
 
@@ -792,8 +817,8 @@ class Tunnel(ChunkParser):
         # conn is None means we have no back-end.
         # conn is False means authentication is required.
         if not self.SendChunked('SID: %s\r\n\r\n%s' % (sid,
-                                HTTP_Unavailable('be', proto, host,
-                                  frame_url=self.conns.config.error_url,
+                                self.HTTP_Unavail(
+                                  self.conns.config, 'be', proto, host,
                                   code=code
                                 )), just_buffer=True):
           return False, False
@@ -1058,7 +1083,7 @@ class UserConn(Selectable):
 
     if ':' in host: host, port = host.split(':', 1)
     self.proto = proto
-    self.host = host
+    self.host = StripEncodedIP(host)
 
     # If the listening port is an alias for another...
     if int(on_port) in conns.config.server_portalias:
@@ -1437,15 +1462,15 @@ class UnknownConn(MagicProtocolParser):
           chost, cport = connect_parser.path.split(':', 1)
 
           cport = int(cport)
-          chost = chost.lower()
+          chost = StripEncodedIP(chost.lower())
           sid1 = ':%s' % chost
           sid2 = '-%s:%s' % (cport, chost)
           tunnels = self.conns.tunnels
 
           if not self.conns.config.CheckClientAcls(self.address, conn=self):
-            self.Send(HTTP_Unavailable('fe', 'raw', chost,
-                                       code=403, status='Forbidden',
-                                       frame_url=self.conns.config.error_url),
+            self.Send(self.HTTP_Unavail(
+                        self.conns.config, 'fe', 'raw', chost,
+                        code=403, status='Forbidden'),
                       try_flush=True)
             return False
 
@@ -1509,7 +1534,7 @@ class UnknownConn(MagicProtocolParser):
       if not self.host:
         hosts = self.parser.Header('Host')
         if hosts:
-          self.host = hosts[0].lower()
+          self.host = StripEncodedIP(hosts[0].lower())
         else:
           self.Send(HTTP_Response(400, 'Bad request',
                     ['<html><body><h1>400 Bad request</h1>',
@@ -1521,7 +1546,7 @@ class UnknownConn(MagicProtocolParser):
 
       if self.parser.path.startswith(MAGIC_PREFIX):
         try:
-          self.host = self.parser.path.split('/')[2]
+          self.host = StripEncodedIP(self.parser.path.split('/')[2])
           if self.parser.path.endswith('.json'):
             self.proto = 'probe.json'
           else:
@@ -1537,9 +1562,9 @@ class UnknownConn(MagicProtocolParser):
             self.proto = 'websocket'
 
       if not self.conns.config.CheckClientAcls(self.address, conn=self):
-        self.Send(HTTP_Unavailable('fe', self.proto, self.host,
-                                   code=403, status='Forbidden',
-                                   frame_url=self.conns.config.error_url),
+        self.Send(self.HTTP_Unavail(
+                    self.conns.config, 'fe', self.proto, self.host,
+                    code=403, status='Forbidden'),
                   try_flush=True)
         return False
 
@@ -1557,10 +1582,10 @@ class UnknownConn(MagicProtocolParser):
           self.Send(HTTP_NoFeConnection(self.proto),
                     try_flush=True)
         else:
-          self.Send(HTTP_Unavailable('fe', self.proto, self.host,
-                                     frame_url=self.conns.config.error_url,
-                                     overloaded=self.conns.config.Overloaded()),
-                    try_flush=True)
+          self.Send(self.HTTP_Unavail(
+                        self.conns.config, 'fe', self.proto, self.host,
+                        overloaded=self.conns.config.Overloaded()
+                    ), try_flush=True)
         return False
 
     # We are done!
