@@ -4,7 +4,7 @@ Protocol parsers for classifying incoming network connections.
 ##############################################################################
 LICENSE = """\
 This file is part of pagekite.py.
-Copyright 2010-2016, the Beanstalks Project ehf. and Bjarni Runar Einarsson
+Copyright 2010-2017, the Beanstalks Project ehf. and Bjarni Runar Einarsson
 
 This program is free software: you can redistribute it and/or modify it under
 the terms of the  GNU  Affero General Public License as published by the Free
@@ -20,6 +20,8 @@ You should have received a copy of the GNU Affero General Public License
 along with this program.  If not, see: <http://www.gnu.org/licenses/>
 """
 ##############################################################################
+import re
+
 from pagekite.compat import *
 from pagekite.common import *
 import pagekite.logging as logging
@@ -29,6 +31,14 @@ HTTP_METHODS = ['OPTIONS', 'CONNECT', 'GET', 'HEAD', 'POST', 'PUT', 'TRACE',
                 'LOCK', 'UNLOCK', 'PING', 'PATCH']
 HTTP_VERSIONS = ['HTTP/1.0', 'HTTP/1.1']
 
+EMBEDDED_IP_RE = re.compile(r'^0x(?:[0-9a-fA-F]{8,8}|[0-9a-fA-F]{32,32})--')
+
+
+def StripEncodedIP(domain):
+  if domain:
+    return EMBEDDED_IP_RE.sub('', str(domain))
+  return domain
+
 
 class BaseLineParser(object):
   """Base protocol parser class."""
