@@ -4,7 +4,7 @@ Selectables are low level base classes which cooperate with our select-loop.
 ##############################################################################
 LICENSE = """\
 This file is part of pagekite.py.
-Copyright 2010-2016, the Beanstalks Project ehf. and Bjarni Runar Einarsson
+Copyright 2010-2017, the Beanstalks Project ehf. and Bjarni Runar Einarsson
 
 This program is free software: you can redistribute it and/or modify it under
 the terms of the  GNU  Affero General Public License as published by the Free
@@ -21,6 +21,7 @@ along with this program.  If not, see: <http://www.gnu.org/licenses/>
 """
 ##############################################################################
 import errno
+import re
 import struct
 import threading
 import time
@@ -28,6 +29,7 @@ import zlib
 
 from pagekite.compat import *
 from pagekite.common import *
+from pagekite.proto.proto import HTTP_Unavailable
 import pagekite.logging as logging
 import pagekite.compat as compat
 import pagekite.common as common
@@ -72,6 +74,8 @@ class Selectable(object):
     self.fd = None
 
     try:
+      if bind and bind[0] and re.match(r'^\d+\.\d+\.\d+\.\d+$', bind[0]):
+        raise ValueError('Avoid INET6 for IPv4 hosts')
       self.SetFD(fd or rawsocket(socket.AF_INET6, socket.SOCK_STREAM), six=True)
       if bind:
         self.fd.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
@@ -369,17 +373,20 @@ class Selectable(object):
     except IOError, err:
       if err.errno not in self.HARMLESS_ERRNOS:
         self.LogDebug('Error reading socket: %s (%s)' % (err, err.errno))
+        common.DISCONNECT_COUNT += 1
         return False
       else:
         return True
     except (SSL.Error, SSL.ZeroReturnError, SSL.SysCallError), err:
       self.LogDebug('Error reading socket (SSL): %s' % err)
+      common.DISCONNECT_COUNT += 1
       return False
     except socket.error, (errno, msg):
       if errno in self.HARMLESS_ERRNOS:
         return True
       else:
         self.LogInfo('Error reading socket: %s (errno=%s)' % (msg, errno))
+        common.DISCONNECT_COUNT += 1
         return False
 
     self.last_activity = now
@@ -455,7 +462,9 @@ class Selectable(object):
     if sending:
       try:
         want_send = self.write_retry or min(len(sending), SEND_MAX_BYTES)
-        for try_wait in (0, 0, 0.1, 0.2, 0.5):
+        sent_bytes = None
+        # Try to write for up to 5 seconds before giving up
+        for try_wait in (0, 0, 0.1, 0.2, 0.2, 0.2, 0.3, 0.5, 0.5, 1, 1, 1, 0):
           try:
             sent_bytes = self.fd.send(sending[:want_send])
             if logging.DEBUG_IO:
@@ -467,11 +476,18 @@ class Selectable(object):
           except (SSL.WantWriteError, SSL.WantReadError), err:
             if logging.DEBUG_IO:
               print '=== WRITE SSL RETRY: =[%s: %s bytes]==' % (self, want_send)
-            if try_wait: time.sleep(try_wait)
+            if try_wait:
+              time.sleep(try_wait)
+        if sent_bytes is None:
+          self.LogInfo('Error sending: Too many SSL write retries')
+          self.ProcessEofWrite()
+          common.DISCONNECT_COUNT += 1
+          return False
       except IOError, err:
         if err.errno not in self.HARMLESS_ERRNOS:
           self.LogInfo('Error sending: %s' % err)
           self.ProcessEofWrite()
+          common.DISCONNECT_COUNT += 1
           return False
         else:
           if logging.DEBUG_IO:
@@ -481,6 +497,7 @@ class Selectable(object):
         if errno not in self.HARMLESS_ERRNOS:
           self.LogInfo('Error sending: %s (errno=%s)' % (msg, errno))
           self.ProcessEofWrite()
+          common.DISCONNECT_COUNT += 1
           return False
         else:
           if logging.DEBUG_IO:
@@ -489,6 +506,7 @@ class Selectable(object):
       except (SSL.Error, SSL.ZeroReturnError, SSL.SysCallError), err:
         self.LogInfo('Error sending (SSL): %s' % err)
         self.ProcessEofWrite()
+        common.DISCONNECT_COUNT += 1
         return False
       except AttributeError:
         # This has been seen in the wild, is most likely some sort of
@@ -575,6 +593,26 @@ class Selectable(object):
     self.read_eof = self.write_eof = True
     return True
 
+  def HTTP_Unavail(self, config, where, proto, host, **kwargs):
+    kwargs['frame_url'] = config.error_url
+    if self.fd:
+      kwargs['relay_sockname'] = self.fd.getsockname()
+
+    # Do we have a more specific error URL for this domain? This is a
+    # white-label feature, for folks not wanting to hit the PageKite.net
+    # servers at all. In case of a match, we also disable mention of
+    # PageKite itself in the HTML boilerplate.
+    dparts = host.split('.')
+    while dparts:
+      fu = config.error_urls.get('.'.join(dparts), None)
+      if fu is not None:
+        kwargs['frame_url'] = fu
+        kwargs['advertise'] = False
+        break
+      dparts.pop(0)
+
+    return HTTP_Unavailable(where, proto, host, **kwargs)
+
 
 class LineParser(Selectable):
   """A Selectable which parses the input as lines of text."""
